import * as turf from '@turf/turf';



// Because Web Workers run in their own global scope, they are responsible for
// ensuring its dependencies are available.  `importScripts` is used to import
// scripts into the global scope.  `importScripts` runs synchronously.

// Logging works in web-workers.  The message shows in the main thread's console.
console.log('[TurfWebWorker]: Loading Dependencies...');

//importScripts('https://cdn.jsdelivr.net/npm/@turf/turf@5/turf.min.js');
console.log('[TurfWebWorker]: Turf JS loaded.');

function onError(evt) {
  console.error(evt);
}

/**
 * @param {MessageEvent} evt - The message event.
 * @param {Object} evt.data - The message-based data.
 * @param {string} evt.data.type - The type of the message sent.  Must be one of
 *   `'request'`.  (Only one type expected at the moment.)
 * @param {*} evt.data.... - Remaining data based on type of message.
 */
function onMessage(evt) {
  var message = evt.data;

  if (message.type === 'request') {
	handleRequest(message);
  } else {
	console.warn('[TurfWebWorker]: Received a message of unexpected type: "' + message.type + '".');
  }
}

/**
 * @param {Object} req - A request message.
 * @param {string} req.action - The action of the request.
 * @param {Object} req.params - The parameters based on the action.
 */
function handleRequest(req) {
  var action, params;

  action = req.action;
  params = req.params;

  console.log('[TurfWebWorker]: Request received: action: "' + action + '" params: "' + JSON.stringify(params) + '"');

  switch (action) {
	case 'randomData':
	  var randomData, geometry, count;

	  geometry = params.geometry;
	  count = params.count;

	  if (geometry === 'point') {
		randomData = turf.randomPoint(count);
	  } else if (geometry === 'line') {
		randomData = turf.randomLineString(count);
	  } else if (geometry === 'polygon') {
		randomData = turf.randomPolygon(count);
	  } else {
		console.warn('Geometry type ', geometry, ' not recognized');
	  }

	  postMessage({
		type: 'response',
		action: 'randomData',
		data: randomData
	  });
	  break;

	case 'buffer':
	  var buffered, distance, geojson, units;

	  distance = params.distance;
	  geojson = params.geojson;
	  units = params.units;

	  buffered = turf.buffer(geojson, distance, { units: units });

	  postMessage({
		type: 'response',
		action: 'buffer',
		data: buffered
	  });
	  break;
	default:
	  console.warn('[TurfWebWorker]: Received an unexpected request: "' + action + '".');
  }
}

addEventListener('error', onError);
addEventListener('message', onMessage);